
unit FileIo;

interface
uses
       MemTypes,
      QuickDraw,
         OsIntf,
       ToolIntf,
       PackIntf;





function Exists(vRefNum: integer; fname: Str255): boolean;
procedure MakeFile     (vRefNum: integer; fname: Str255; Cre, Typ: OsType);
procedure OpenOut      (vRefNum: integer; fname: Str255);
procedure OpenOutRsrc  (vRefNum: integer; fname: Str255);
procedure PutChar      (c: char);
procedure CloseOut;
procedure OpenIn       (vol: integer; fn: Str255);
procedure OpenInRsrc   (vol: integer; fn: Str255);
procedure GetChar      (var c: char);
procedure GetLine      (var S: Str255);
procedure PutLine      (S: Str255);
procedure CloseIn;

var
      FileError: integer;
          InRef,
         OutRef: integer;
        BufSize: longint;
        FileEOF: boolean;





implementation

const
         debug = true;
    MaxBufSize = $4000;

type
            CA = packed array [ 1..1 ] of char;
       CharPtr = ^CA;
          CHan = ^CharPtr;

var
        InCount,
       OutCount,
          InPtr: longint;
          InBuf,
         OutBuf: CHan;
           OVol: integer;
{$I Options.p }






procedure MakeBuf(var Buf: Chan);
   
   var
                 I: integer;
                 c: Char;
               Big,
              Grow,
               MBF: longint;
   
   begin { MakeBuf}
      MaxApplZone;
      Big := MaxMem(Grow);
      BufSize := (Big - 10000) div 2;
      MBF := MaxBufSize;
      if BufSize > MBF then
         BufSize := MBF;
      Buf := Chan(NewHandle(BufSize));
      if Buf = NIL
      then
         FileError := -987
      else
         FileError := NoErr;
   end {MakeBuf};





function OpenX(fn: Str255; V: integer; var Ref: integer; Perm: integer; Buf: Ptr;
   Rsrc: boolean): integer;
   
   var
                PB: ParamBlockRec;
               Err: OsErr;
   
   begin { OpenX}
      with PB do
         begin
            ioCompletion := NIL;
            ioNamePtr := StringPtr(@ Fn);
            ioVRefNum := V;
            ioVersNum := 0;
            ioPermssn := Perm;
            ioMisc := Buf;
         end;
      if Rsrc
      then
         Err := PBOpenRF(ParmBlkPtr(@ PB), false)
      else
         Err := PBOpen(ParmBlkPtr(@ PB), false);
      Ref := PB.ioRefNum;
      OpenX := Err;
   end {OpenX};





function Exists{(vRefNum: integer; fname: Str255): boolean};
   
   var
                Ex: integer;
               Err,
               Ref: integer;
   
   begin { Exists}
      Ex := OpenX(fname, vRefNum, Ref, fsRdPerm, NIL, false);
      if Ex = NoErr
      then
         begin
            Err := FSClose(Ref);
            Exists := true;
         end
      else
         Exists := false;
   end {Exists};





procedure OpenFile(vRefNum: integer; var fname: Str255; R: boolean);
   
   begin { OpenFile}
      MakeBuf(OutBuf);
      if FileError = NoErr then
         begin
            if not Exists(vRefNum, fname) then
               FileError := Create(fname, vRefNum, '????', '????');
            if FileError = NoErr then
               FileError := OpenX(fname, vRefNum, OutRef, fsWrPerm, NIL, R);
         end;
      OutCount := 0;
      OVol := vRefNum;
   end {OpenFile};





procedure MakeFile{(vRefNum: integer; fname: Str255; Cre, Typ: OsType)};
   
   begin { MakeFile}
      if Exists(vRefNum, fname)
      then
         FileError := FSDelete(fname, vRefNum)
      else
         FileError := NoErr;
      if FileError = NoErr then
         FileError := Create(fname, vRefNum, Cre, Typ);
   end {MakeFile};





procedure OpenOut {(vRefNum: integer; fname: Str255 )};
   
   begin { OpenOut}
      OpenFile(vRefNum, fname, false);
   end {OpenOut};





procedure OpenOutRsrc {(vRefNum: integer; fname: Str255 )};
   
   begin { OpenOutRsrc}
      OpenFile(vRefNum, fname, true);
   end {OpenOutRsrc};





procedure FlushOut;
   
   var
                OP: Ptr;
   
   begin { FlushOut}
      if OutCount > 0 then
         begin
            HLock(Handle(OutBuf));
            OP := Ptr(OutBuf ^);
            FileError := FSWrite(OutRef, OutCount, OP);
            HUnLock(Handle(OutBuf));
            OutCount := 0;
         end;
   end {FlushOut};





procedure PutChar {(c: char)};
   
   begin { PutChar}
      OutCount := OutCount + 1;
      OutBuf ^^ [ OutCount ] := c;
      if OutCount >= BufSize then
         FlushOut;
   end {PutChar};





procedure xClose(var F: integer);
   
   begin { xClose}
      if F <> 0 then
         begin
            FileError := FSClose(F);
            F := 0;
         end;
   end {xClose};





procedure CloseIn;
   
   begin { CloseEm}
      xClose(InRef);
      DisposHandle(Handle(InBuf));
   end {CloseEm};





procedure CloseOut;
   
   begin { CloseEm}
      if OutRef <> 0 then
         begin
            FlushOut;
            if FileError = NoErr then
               begin
                  xClose(OutRef);
                  if FileError = NoErr then
                     FileError := FlushVol(NIL, OVol);
                  DisposHandle(Handle(OutBuf));
               end;
         end;
   end {CloseEm};





procedure OpenIn {(vol: integer; fn: Str255)};
   
   begin { OpenIn}
      MakeBuf(InBuf);
      if FileError = NoErr then
         begin
            FileError := OpenX(fn, vol, InRef, fsRdPerm, NIL, false);
            InCount := 0;
            InPtr := 0;
         end;
   end {OpenIn};





procedure OpenInRsrc {(vol: integer; fn: Str255)};
   
   begin { OpenInRsrc}
      MakeBuf(InBuf);
      if FileError = NoErr then
         begin
            FileError := OpenX(fn, vol, InRef, fsRdPerm, NIL, true);
            InCount := 0;
            InPtr := 0;
         end;
   end {OpenInRsrc};





procedure FillBuf;
   
   var
                IP: Ptr;
   
   begin { FillBuf}
      InCount := BufSize;
      HLock(Handle(InBuf));
      IP := Ptr(InBuf ^);
      FileError := FSRead(InRef, InCount, IP);
      HUnLock(Handle(InBuf));
      if FileError = eofErr then
         FileError := NoErr;
      InPtr := 0;
   end {FillBuf};





procedure GetChar {(var c: char)};
   
   begin { GetChar}
      if InCount <= 0 then
         FillBuf;
      FileEOF := InCount <= 0;
      if not FileEOF then
         begin
            InCount := InCount - 1;
            InPtr := InPtr + 1;
            c := InBuf ^^ [ inptr ];
         end;
   end {GetChar};





procedure GetLine{(var S: Str255)};
   
   var
                 C: char;
   
   begin { GetLine}
      S := '';
      repeat
         GetChar(C);
         if not FileEOF then
            if C <> chr(13) then
               S := concat(S, C);
      until FileEOF or (C = Chr(13));
   end {GetLine};





procedure PutLine{(S: Str255)};
   
   var
                 i: integer;
   
   begin { PutLine}
      for i := 1 to Length(S) do
         PutChar(S[ i ]);
      PutChar(chr(13));
   end {PutLine};





end.
