
unit MacTCPIntf;

interface
uses
       MemTypes,
      QuickDraw,
         OsIntf,
       ToolIntf,
       PackIntf,
     PasLibIntf,
        TCPDefs;

type
           int = longint;
   LongNatural = 0..$7FFFFFFF;





function TCPinit       (HostName: Str255): OsErr;
{
	Looks up HostName, returns NoErr if Host can be found.
	Must be called before anybody else.
}
function TCPconnect    (Service: Str255; WellKnownPort: int): OsErr;
{
	Opens a connection to Service on the aforelookedup machine.
	For now, you must also pass in a static port number in WellKnownPort.
}
function TCPclose:      OsErr;
{
	*Must* be called at program termination if TCPConnect ever returns NoErr.
}
function TCPRClose:     OsErr;  { Closes the name resolver }
function TCPRead       (Buf: CharsPtr; MaxLength, RequestLen: LongNatural;
   StopChar: char): OsErr;
{
	Reads an arbitrary number of bytes from the opened connection.
	MaxLength is Sizeof( Buf ).
	RequestLen is the number of chars to be read.
	Hangs forever if that number is not available!!
	if RequestLen = 0 then it reads until encounters StopChar.
	StopChar read but not placed in Buf.
	Returns number of chars read, or < 0 if error.
}
function TCPWrite      (Buf: Ptr; Len: LongNatural): OsErr;
{
	Writes an arbitrary number of bytes from the opened connection.
	Returns number of chars written, or < 0 if error.
}
function TCPReadChar   (var c: char): OsErr;
function TCPReadString (var S: Str255): OsErr;
function TCPWriteString(S: Str255): OsErr;
function TCPWriteStringV       (var S: Str255): OsErr;
function TCPServer:     OsErr;
function TCPFlushBuffer:        OsErr;
function TCPTimeOuts   (NameLookupTimeoutSeconds, ConnectTimeOutSeconds,
   IOTimeOutSeconds: integer): OsErr;
function TCPIdle       (P: Ptr): OsErr;

var
 ResolverIsOpen: boolean;
 OurIPAddress: longint;





implementation
{$SETC  Debug := 0 }

const
 ReadUntilChar = 0;
            CR = 13;
            LF = 10;
         Debug = false;
      BuffSize = $3000;
 Synchronously = false;
  DNameTimeOut = 20;
  DWaitSeconds = 180;
  DConnSeconds = 30;
     InBufSize = 512;
      MailPort = 25;
      OBufSize = 2048;
	  ResPW = $92939495;
	  ResTabSize = 5;

type
          PAC2 = packed array [ 0..10000000 ] of char;
          PPtr = ^PAC2;
    RefNumType = integer;
 IPAddressType = longint;
 InBufPtrRange = 0..InBufSize;
OutBufPtrRange = 0..OBufSize;
       Natural = 0..MaxInt;
   StringIndex = 1..255;
   ShortStr = string[40];
   ResTabRec = record Name: ShortStr; IP: IPAddressType; end;
   ResTabA = array[1..ResTabSize] of ResTabRec;

var
TheTCPDriversRefNum: RefNumType;
            Err: OsErr;
TheStreamPointer: Ptr;
  HostIPAddress
         : IPAddressType;
NameLookupComplete,
  OutputPending: boolean;
       InBufPtr: PPtr;
          inptr,
        incount: InBufPtrRange;
       OBufBase: Ptr;
      OBufAvail,
        OBufPtr: OutBufPtrRange;
           CRLF: packed array [ 1..2 ] of char;
    ConnSeconds,
    WaitSeconds,
    NameTimeOut: integer;
       HookProc: Ptr;
	   ResolverPW: longint;
	   ResTab: ResTabA;
	   
{$R-}
{$D+}





procedure PasToC(S: Str255; var c: CStr);
   
   var
                 i: StringIndex;
   
   begin { PasToC}
      for i := 1 to length(S) do
         c [ i ] := S[ i ];
      c [ Length(S) +1 ] := chr(0);
   end {PasToC};





procedure CToP(c: CStr; var S: Str255);
   
   var
                 i: StringIndex;
   
   begin { CToP}
      I := 1;
      while (i <= 255) and (c [ i ] <> chr(0)) do
         begin
            S[ i ] := c [ i ];
            i := i + 1;
         end;
{$R-}
      S[ 0 ] := Chr(i - 1);
{$R-}
   end {CToP};





procedure NameProc(HI: HostInfo; UData: Ptr);
   
   var
                 N: Str255;
                 i: 1..5;
   
   begin { NameProc}
      CToP(Hi.CName, N);
      with HI do
         begin
            HostIPAddress := addr[ 1 ];
            NameLookupComplete := true;
         end;
   end {NameProc};





function TCPIdle(P: Ptr): OsErr;
   
   begin { TCPIdle}
      HookProc := P;
      TCPIdle := NoErr;
   end {TCPIdle};





procedure SpinWheels;
   
   
   
   
   
   procedure CallProc(P: Ptr); inline $4E75;
   
   
   begin { SpinWheels}
      if HookProc <> NIL then
         CallProc(HookProc);
   end {SpinWheels};





function TCPinit(HostName: Str255): OsErr;
   
   
   
   
   
   procedure OpenLoDriver;
      
      begin { OpenLoDriver}
         Err := OpenDriver('.ipp', TheTCPDriversRefNum);
      end {OpenLoDriver};
   
   
   
   
   
   procedure NewResolver(InName: Str255);
      
      var
                   HI: HostInfo;
                 Name: CStr;
              EndTime: longint;
                 Err2: OsErr;
      
	  
	  procedure InitResTab;
	  var i: integer;
	  begin
	     if ResolverPW <> ResPW then
		 begin
		    for i := 1 to ResTabSize do
			with ResTab[ i ] do
			begin
			    Name := '';
				IP := 0;
		    end;
			ResolverPW := ResPW;
		end;
	 end;
	  
	  
	  procedure CheckResTab;
	  var i: integer;
	  begin HostIPAddress := 0; 
	     for i := 1 to ResTabSize do 
		 if HostIPAddress = 0 then with ResTab[ i ] do
		 if InName = Name then HostIPAddress := IP;
	  end;
	  
	  procedure ResEnter( N: ShortStr; A: IPAddressType );
	  var i: integer; found: boolean;
	  begin
	     for i := 1 to ResTabSize do with ResTab[ i ] do
		 begin if Name = N then begin Name := '';IP := 0; end;
		 end;
		 found := false;
	     for i := 1 to ResTabSize do 
		 if not found then with ResTab[ i ] do
		 begin if Name = '' then begin Name := N; IP := A; found := true;end;
		 end;
		 if not found then with ResTab[1] do begin
		 Name := N; IP := A;end;
		 
	end;
	
	
      begin { NewResolver}
         ResolverIsOpen := false;
		 InitResTab;
		 CheckResTab;
         if HostIPAddress = 0
         then
            begin
               Err := GRGOpenResolver('');
               if Err = NoErr then
                  begin
                     ResolverIsOpen := true;
                     NameLookupComplete := false;
                     HostIPAddress := 0;
                     PasToC(InName, Name);
                     Err := StrToAddr(Name, HI, Ptr(@ NameProc), NIL);
                     if Err = cachefault
                     then
                        begin
                           EndTime := TickCount + NameTimeOut * 60;
                           repeat
                              SpinWheels;
                           until NameLookupComplete or (TickCount > EndTime);
                        end
                     else
                        begin
                           if Err = NoErr then
                              HostIPAddress := Hi.Addr[ 1 ];
                        end;
                     Err2 := TCPRClose;
                  end;
               if HostIPAddress = 0
               then
                  Err := -9876
               else
                  begin
                     Err := NoErr;
                     ResEnter( InName, HostIPAddress );
                  end;
            end;
      end {NewResolver};
   
   
   
   
   
   
   begin { TCPinit}
      TheStreamPointer := NIL;
      OBufBase := NIL;
      InBufPtr := NIL;
      HookProc := NIL;
      OBufAvail := 0;
      OBufPtr := 0;
      CRLF[ 1 ] := Chr(CR);
      CRLF[ 2 ] := Chr(LF);
      NameTimeOut := DNameTimeOut;
      WaitSeconds := DWaitSeconds;
      ConnSeconds := DConnSeconds;
      OpenLoDriver;
      if Err = NoErr then
         NewResolver(HostName);
      TCPInit := Err;
   end {TCPinit};





function DoControl(var PB: TCPIoPB): OsErr;
   
   const
          Waiting = 1;
   
   var
             async: boolean;
               Err: OsErr;
   
   begin { DoControl}
      PB.ioCompletion := NIL;
      async := HookProc <> NIL;
	  async := false;
      Err := PBControl(@ PB, async);
      if async then
         begin
            repeat
               SpinWheels;
            until PB.IoResult <> Waiting;
            Err := PB.ioResult;
         end;
      DoControl := Err;
   end {DoControl};
{---
procedure ASR(TheStream: Ptr; EventCode: integer; UserDataPtr: Ptr; TerminReason:
   integer; ICMPPTr: Ptr);
   
   begin 
   end;
---}





function TCPRClose: OsErr;
   
   var
               Err: OsErr;
   
   begin { TCPRClose}
      if ResolverIsOpen
      then
         begin
            Err := CloseResolver;
            ResolverIsOpen := Err <> NoErr;
         end
      else
         Err := NoErr;
      TCPRClose := Err;
   end {TCPRClose};





function TCPconnect(Service: Str255; WellKnownPort: int): OsErr;
   
   type
              PBP = ^TCPIOPB;
   
   var
                PB: TCPIoPB;
                 P: PBP;
                 T: Str255;
               IPA: CStr;
   
   
   
   
   
   procedure TCPOpenStream;
      
      var
                   PB: TCPIoPB;
                  Buf: Ptr;
      
      
      
      
      
      procedure AllocateBuffers;
         
         begin { AllocateBuffers}
            TheStreamPointer := PB.tcpStream;
            OBufBase := NewPtr(OBufSize);
            if OBufBase = NIL
            then
               Err := -17
            else
               begin
                  OBufAvail := OBufSize;
                  OBufPtr := 0;
                  InBufPtr := PPtr(NewPtr(InBufSize));
                  if InBufPtr = NIL then
                     Err := -17;
               end;
         end {AllocateBuffers};
      
      
      
      
      
      
      begin { TCPOpen}
         Buf := Pointer(NewPtr(BuffSize));
         if Buf = NIL
         then
            Err := -17
         else
            begin
               with PB, Create do
                  begin
                     ioCompletion := NIL;
                     ioCRefNum := TheTCPDriversRefNum;
                     ioVRefNum := 0;
                     rcvBuff := Buf;
                     rcvBuffLen := BuffSize;
                     CsCode := TCPCreate;
                     notifyProc := NIL {Pointer(@ ASR)};
                     userDataPtr := NIL;
                     tcpStream := NIL;
                  end;
               Err := DoControl(PB);
               if Err = NoErr then
                  AllocateBuffers;
            end;
      end {TCPOpen};
   
   
   
   
   
   
   begin { TCPOpenConnection}
      TCPOpenStream;
      OutputPending := false;
      incount := 0;
      if Err = NoErr then
         begin
            P := PBP(@ PB);
            with P ^, Open do
               begin
                  ioCRefNum := TheTCPDriversRefNum;
                  ioVRefNum := 0;
                  ioCompletion := NIL;
                  csCode := TCPActiveOpen;
                  tcpStream := TheStreamPointer;
                  ulpTimeOutValue := ConnSeconds;
                  ulpTimeOutAction := 1;
                  validityFlags := timeoutValue + timeoutAction;
                  commandTimeOutValue := ConnSeconds;
                  RemoteHost := HostIPAddress;
                  RemotePort := WellKnownPort;
                  LocalPort := 0;
                  tosFlags := 0;
                  precedence := 0;
                  dontfrag := false;
                  timeToLive := 0;
                  security := 0;
                  optioncnt := 0;
                  userDataPtr := NIL;
               end;
            Err := DoControl(PB);
         end;
      TCPConnect := Err;
	  OurIPAddress := P^.Open.LocalHost;
   end {TCPconnect};





function TCPWriteLow(Buf: Ptr; Len: Natural; PushOut: boolean): OsErr;
   
   const
           maxwds = 2;
   
   type
          WDSType = array [ 1..maxwds ] of WDSEntry;
   
   var
                PB: TCPIOPB;
               WDS: WDSType;
   
   begin { TCPWriteLow}
      with WDS[ 1 ] do
         begin
            ptr := BPtr(Buf);
            Length := Len;
         end;
      with WDS[ 2 ] do
         begin
            ptr := NIL;
            Length := 0;
         end;
      with PB, send do
         begin
            ioCompletion := NIL;
            ioCRefNum := TheTCPDriversRefNum;
            ioVRefNum := 0;
            csCode := TCPSend;
            tcpStream := TheStreamPointer;
            ulpTimeoutValue := WaitSeconds;
            ulpTimeoutAction := 0;
            validityFlags := timeoutValue;
            pushFlag := PushOut;
            urgentflag := false;
            wdsptr := Pointer(@ WDS);
         end;
      Err := DoControl(PB);
      TCPWriteLow := Err;
   end {TCPWriteLow};





function TCPFlushBuffer: OsErr;
   
   begin { TCPFlushBuffer}
      if OBufPtr > 0
      then
         begin
            TCPFlushBuffer := TCPWriteLow(OBufBase, OBufPtr, true);
            OBufAvail := OBufSize;
            OBufPtr := 0;
         end
      else
         TCPFlushBuffer := NoErr;
   end {TCPFlushBuffer};





function TCPWrite(Buf: Ptr; Len: LongNatural): OsErr;
   
   var
               Err: OsErr;
                ip,
                 L: LongNatural;
   
   begin { TCPWrite}
      Err := NoErr;
      ip := 0;
      repeat
         L := Len;
         if L > OBufAvail then
            L := OBufAvail;
         BlockMove(Ptr(Ord4(Buf) +ip), Ptr(Ord4(OBufBase) +OBufPtr), L);
         OBufPtr := OBufPtr + L;
         ip := ip + L;
         OBufAvail := OBufAvail - L;
         if OBufAvail <= 0 then
            Err := TCPFlushBuffer;
         Len := Len - L;
      until (Len <= 0) or (Err <> NoErr);
      TCPWrite := Err;
   end {TCPWrite};





function TCPWriteStringV(var S: Str255): OsErr;
   
   var
               Len: StringIndex;
               Err: OsErr;
   
   begin { TCPWriteString}
      Len := Length(S);
      Err := TCPWrite(Ptr(Ord4(@ S) +1), Len);
      if Err = NoErr then
         Err := TCPWrite(@ CRLF, 2);
      TCPWriteStringV := Err;
   end {TCPWriteString};





function TCPWriteString(S: Str255): OsErr;
   
   var
               Len: StringIndex;
               Err: OsErr;
   
   begin { TCPWriteString}
      Len := Length(S);
      Err := TCPWrite(Ptr(Ord4(@ S) +1), Len);
      if Err = NoErr then
         Err := TCPWrite(@ CRLF, 2);
      TCPWriteString := Err;
   end {TCPWriteString};





function TCPClose: OsErr;
   
   var
                PB: TCPIOPB;
   
   
   procedure CloseConn;
       begin 
         Err := TCPFlushBuffer;
               with PB, Close do
                  begin
                     ioCRefNum := TheTCPDriversRefNum;
                     ioVRefNum := 0;
                     ioCompletion := NIL;
                     csCode := TCPCloze;
					 ulpTimeOutValue := 10;
					 ulpTimeOutAction := 0;
                     validityFlags := timeoutValue;
                     tcpStream := TheStreamPointer;
                  end;
               Err := DoControl(PB);
      end;
   
  
   
   procedure CloseTheStream;
      
      begin { CloseTheStream}
         Err := TCPFlushBuffer;
               with PB do
                  begin
                     ioCRefNum := TheTCPDriversRefNum;
                     ioVRefNum := 0;
                     ioCompletion := NIL;
                     csCode := TCPRelease;
                     tcpStream := TheStreamPointer;
                  end;
               Err := DoControl(PB);
               if Err = NoErr then
                  begin
                     DisposPtr(PB.Create.rcvBuff);
                     Err := MemError;
                     TheStreamPointer := NIL;
                  end;
      end {CloseTheStream};
   
   
   
   
   
   procedure DisposPtrNIL(var P: Ptr);
      
      begin { DisposPtrNIL}
         if P <> NIL then
            begin
               DisposPtr(P);
               Err := MemError;
               P := NIL;
            end;
      end {DisposPtrNIL};
   
   
   
   
   
   
   begin { TCPClose}
      Err := NoErr;
      if TheStreamPointer <> NIL then
	  begin
	     CloseConn;
         CloseTheStream;
	  end;
      DisposPtrNIL(OBufBase);
      DisposPtrNIL(Ptr(InBufPtr));
      TCPClose := Err;
   end {TCPclose};





function TCPDataAvailable: LongNatural;
   
   var
                PB: TCPIoPB;
   
   begin { TCPDataAvailable}
      with PB, Status do
         begin
            ioCompletion := NIL;
            ioCRefNum := TheTCPDriversRefNum;
            ioVRefNum := 0;
            CsCode := TCPStatus;
            tcpStream := TheStreamPointer;
         end;
      Err := DoControl(PB);
      if Err = NoErr
      then
         with PB, Status do
            TCPDataAvailable := amtUnreadData
      else
         TCPDataAvailable := -1;
   end {TCPDataAvailable};





function FixedRead(Buf: BPtr; Len: Natural): OsErr;
   
   var
                PB: TCPIOPB;
   
   begin { FixedRead}
      with PB, receive do
         begin
            ioCompletion := NIL;
            ioCRefNum := TheTCPDriversRefNum;
            ioVRefNum := 0;
            csCode := TCPRcv;
            tcpStream := TheStreamPointer;
            urgentflag := false;
            markflag := false;
            commandTimeoutValue := WaitSeconds;
            rcvBuff := Pointer(Buf);
            rcvBuffLen := Len;
         end;
      Err := DoControl(PB);
      FixedRead := Err;
   end {FixedRead};





function TCPReadChar(var c: char): OsErr;
   
   
   
   
   
   procedure FillBuffer;
      
      var
                Avail: LongNatural;
                  Now,
                  L60,
           FinishTime: longint;
      
      begin { FillBuffer}
         Now := TickCount;
         L60 := 60;
         FinishTime := Now + WaitSeconds * L60;
         repeat
            Avail := TCPDataAvailable;
            if Avail > InBufSize then
               Avail := InBufSize;
            SpinWheels;
         until (Avail > 0) or (TickCount > FinishTime);
         if Avail > 0
         then
            Err := FixedRead(BPtr(InBufPtr), Avail)
         else
            Err := -911;
         incount := Avail { should be Err? };
         inptr := 0;
      end {FillBuffer};
   
   
   
   
   
   
   begin { TCPReadChar}
      Err := NoErr;
      if incount <= 0 then
         FillBuffer;
      if Err >= NoErr
      then
         begin
            c := InBufPtr ^ [ inptr ];
            inptr := inptr + 1;
            incount := incount - 1;
         end
      else
         c := '?';
      TCPReadChar := Err;
   end {TCPReadChar};





function TCPRead(Buf: CharsPtr; MaxLength: LongNatural; RequestLen: LongNatural;
   StopChar: char): OsErr;
   
   type
              PPB = ^TCPIOPB;
   
   
   
   
   
   procedure VarRead;
      
      type
                 pac = packed array [ 1..10000000 ] of char;
                PPtr = ^PAC;
      
      var
               CCount: LongNatural;
                   Ch: char;
      
      begin { VarRead}
         CCount := 0;
         repeat
            Err := TCPReadChar(Ch);
            if Err = NoErr then
               begin
                  if Ch <> StopChar then
                     begin
                        CCount := CCount + 1;
                        Pptr(Buf) ^ [ CCount ] := Ch;
                     end;
               end;
         until (Err < NoErr) or (CCount >= MaxLength) or (Ch = StopChar);
         if Err = NoErr then
            Err := CCount;
      end {VarRead};
   
   
   
   
   
   procedure GetManyBytes;
      
      var
                   Ch: char;
               CCount: LongNatural;
      
      begin { GetManyBytes}
         Err := NoErr;
         CCount := 0;
         while (Err >= NoErr) and (CCount < RequestLen) do
            begin
               Err := TCPReadChar(Ch);
               if Err >= NoErr then
                  begin
                     Pptr(Buf) ^ [ CCount ] := Ch;
                     CCount := CCount + 1;
                  end;
            end;
      end {GetManyBytes};
   
   
   
   
   
   
   begin { TCPRead}
      Err := TCPFlushBuffer;
      if Err = NoErr then
         begin
            if RequestLen > 0
            then
               GetManyBytes
            else
               VarRead;
         end;
      TCPRead := Err;
   end {TCPRead};





function TCPReadString(var S: Str255): OsErr;
   
   var
               Len,
              Len2: OsErr;
             Dummy: packed array [ 1..1 ] of char;
             Loops: integer;
   
   begin { TCPReadString}
      Len := TCPRead(CharsPtr(Ord(@ S) +1), 255, ReadUntilChar, chr(CR));
      if Len >= 0
      then
         begin
            S[ 0 ] := Chr(Len);
            Loops := 0;
            repeat
               Len2 := TCPRead(@ Dummy, 1, 1, '?');
               Loops := Loops + 1;
            until (Dummy[ 1 ] = chr(LF)) or (Loops > 4);
         end
      else
         S := '';
      TCPReadString := Len;
   end {TCPReadString};





function TCPServer: OsErr;
   
   begin { TCPServer}
      TCPServer := -9999;
   end {TCPServer};





function TCPTimeOuts(NameLookupTimeoutSeconds, ConnectTimeOutSeconds,
   IOTimeOutSeconds: integer): OsErr;
   
   begin { TCPTimeOuts}
      NameTimeout := NameLookupTimeoutSeconds;
      ConnSeconds := ConnectTimeOutSeconds;
      WaitSeconds := IOTimeOutSeconds;
      TCPTimeOuts := NoErr;
   end {TCPTimeOuts};





end.
