/*

XprintField 1.1 -- a HyperCard XCMD in MPW C by Joe Zuffoletto
Apple Computer, Inc. 1987
All Rights Reserved.

This XCMD prints the ENTIRE contents of a HyperCard scrollable text field. 
Printed output will have approximately 1" margins on 8-1/2" x 11" paper (the
bottom margin is sometimes wider -- I'm investigating the problem.  A decent
workaround is to send a text height of 16, regardless of the font size). 
Font and font size will be preserved, but not font style.  Instead, the 
field will be printed in plain text, left-justified.  

The call for this routine from HyperTalk is:

     XprintField fType,fName,fFont,fSize,fHeight,fTop,fBot,fLeft,fRight

The parameters are defined as follows:

     fType     The field's type (true (= card) or false (=bkgnd)).
     fName     The name of the field to be printed.
     fFont     The font of the field coded as an integer (see the
               HyperTalk handler).
     fSize     The font size in points.
     fHeight   The field line height in points.
     fTop      Top print margin in points.
     fBot      Bottom print margin in points.
     fLeft     Left print margin in points.
     fRight    Right print margin in points.

To compile and link this XCMD using Macintosh Programmer's Workshop,

     C-q2 {MPW}:XprintField.c

     link -sn Main=XprintField -sn STDIO=XprintField 
          -sn INTENV=XprintField -rt XCMD=5 
          -m XPRINTFIELD {MPW}"XCMDs and XFCNs":XprintField.c.o 
          "{CLibraries}"CRuntime.o 
          "{CLibraries}"CInterface.o 
          -o "Hard Disk":HyperCard:HyperCard

This link directive puts the XCMD in the HyperCard application with
a resource ID of 5. Substitute the name of the stack you want it in.  
To move XCMDs between stacks, use ResEdit.  They can be in an 
individual stack, the Home stack, the HyperCard application, or the 
System File.  */

#include <Types.h>
#include <Memory.h>
#include <OSUtils.h>
#include <ToolUtils.h>
#include <QuickDraw.h>
#include <PrintTraps.h>
#include <TextEdit.h>
#include <Fonts.h>
#include <HyperXCmd.h>

/* Define our margins (1-1/2" top, 3/4" sides and bottom) in pixels. */

/* #define     topMargin  108      Altered 9/1/88 to parameterize fonts*/
/* #define     myMargin    54                                          */

pascal void Debugger() extern 0xA9FF;   

pascal void XprintField(paramPtr)
     XCmdBlockPtr paramPtr;
{
     /* Declare and initialize variables. */
     
     CursHandle     cursH;
     THPrint        printStuff;
     TPPrPort       myPrintPort;
     TEHandle       myFieldTE;
     TPrStatus      *printStatus;
     Handle         memChek,textHandle;
     Ptr            textPtr;
     short          myFirst,myLast,temp;
     short          pageLen,start,finish,numPages,count,i;
     short          fieldFont,fieldSize,fieldHeight;
	 short          marginTop, marginBot, marginLeft, marginRight;
     short          ErrorHandler();
     long           fieldLength,myLen[999];
     Rect           pageRect;
     Str255         fName,fType,fFont,fSize,fHeight;
     Str255         fTop,fBot,fLeft,fRight;
     Boolean        typeFlag;

     /* Set cursor to wristwatch. */

     InitCursor();
     cursH = GetCursor(watchCursor);
     SetCursor((*cursH));

     /* Convert all input parameters to pascal strings for use by other
        XCMD/XFCN glue routines. */

     ZeroToPas(paramPtr,*(paramPtr->params[0]),&fType);
     ZeroToPas(paramPtr,*(paramPtr->params[1]),&fName);
     ZeroToPas(paramPtr,*(paramPtr->params[2]),&fFont);
     ZeroToPas(paramPtr,*(paramPtr->params[3]),&fSize);
     ZeroToPas(paramPtr,*(paramPtr->params[4]),&fHeight);
     ZeroToPas(paramPtr,*(paramPtr->params[5]),&fTop);
     ZeroToPas(paramPtr,*(paramPtr->params[6]),&fBot);
     ZeroToPas(paramPtr,*(paramPtr->params[7]),&fLeft);
     ZeroToPas(paramPtr,*(paramPtr->params[8]),&fRight);

     typeFlag = StrToBool(paramPtr,&fType);
     fieldFont = (short) StrToNum(paramPtr,&fFont);
     fieldSize = (short) StrToNum(paramPtr,&fSize);
     fieldHeight = (short) StrToNum(paramPtr,&fHeight);
     marginTop = (short) StrToNum(paramPtr,&fTop);
     marginBot = (short) StrToNum(paramPtr,&fBot);
     marginLeft = (short) StrToNum(paramPtr,&fLeft);
     marginRight = (short) StrToNum(paramPtr,&fRight);

     /* Allocate a print record. */

     if (printStuff = (THPrint) NewHandle(sizeof(TPrint)))
     {
          MoveHHi((Handle)printStuff); 
          HLock((Handle)printStuff);

          /* Print routine. */

          PrOpen();
          if (PrError() == noErr)
          {
               PrintDefault(printStuff);
               if (PrError() == noErr)
               {
                    /* Get a handle to the text in the HyperCard field. */

                    textHandle = GetFieldByName(paramPtr,typeFlag,&fName);
                    MoveHHi((Handle)textHandle);
                    HLock((Handle)textHandle);
                    textPtr = *textHandle;
                    fieldLength = StringLength(paramPtr,textPtr);

                    /* Open the printing port. */

                    myPrintPort = PrOpenDoc(printStuff,nil,nil);
                    if (PrError() == noErr)
                    {
                         /* Define printing rectangle on the page. Notice that
                            for bottom margin we truncate the partial line. */

                         pageRect = (*printStuff)->rPaper;
                         pageRect.top += marginTop;
                         pageRect.left += marginLeft;
                         pageRect.right -= marginRight;
                         pageRect.bottom -= marginBot;
                         pageRect.bottom -= ((pageRect.bottom - pageRect.top) %
                                            fieldHeight);

                         /* Get the TextEdit handle and lock it. */

                         if (memChek = NewHandle(sizeof(TERec)))
                         {
                              DisposHandle(memChek);
                              myFieldTE = TENew(&pageRect,&pageRect);

                              /* Customize TERec so it matches the 
                                 HyperCard TERec. */
   
                              (*myFieldTE)->txFont = fieldFont;
                              (*myFieldTE)->txSize = fieldSize;
                              (*myFieldTE)->txFace = normal;
                              (*myFieldTE)->lineHeight = fieldHeight;
                              TESetText(textPtr,fieldLength,myFieldTE);
                              TECalText(myFieldTE);

                              /* Image all the pages of the document. */

                              pageLen = (pageRect.bottom - pageRect.top)
                                        / fieldHeight;
                              start = 0;
                              finish = (*myFieldTE)->nLines;
                              numPages = 0;

                              while (start < finish)
                              {
                                  if (finish - start > pageLen)
                                      myLen[numPages++] = (long) ((*myFieldTE)->
                                                  lineStarts[start + pageLen]
                                             - (*myFieldTE)->lineStarts[start]);
                                  else
                                      myLen[numPages] = (long) 
                                                  ((*myFieldTE)->teLength
                                             - (*myFieldTE)->lineStarts[start]);
                                  start = start + pageLen;
                              }

                              myFirst = (*printStuff)->prJob.iFstPage;
                              myLast = (*printStuff)->prJob.iLstPage;
                              if (myLast > numPages + 1)
                                   myLast = numPages + 1;

                              (*printStuff)->prJob.iFstPage = 1;
                              (*printStuff)->prJob.iLstPage = myLast 
                                                              - myFirst + 1;
                              temp = PrValidate(printStuff);

                              /* Print the document. */

                              for (count = myFirst - 1; count < myLast; count++)
                              {
                                   PrOpenPage(myPrintPort,nil);
                                   if (PrError() == noErr)
                                        TextBox(textPtr,myLen[count],
                                                &pageRect,teJustLeft);
                                   PrClosePage(myPrintPort);
                                   textPtr += myLen[count];
                              }
                              HUnlock(textHandle);
                              DisposHandle(textHandle);
                              TEDispose(myFieldTE);
                         }
                         else
                              ErrorHandler();
                    }
                    else
                         ErrorHandler();
                    PrCloseDoc(myPrintPort);
                    if ((*printStuff)->prJob.bJDocLoop = (char) bSpoolLoop)
                         PrPicFile(printStuff,nil,nil,nil,printStatus);
                    if (PrError() != noErr) ErrorHandler();
               }  
               else
                    ErrorHandler();
          }
          else
               ErrorHandler();

          /* Close printer driver and dispose handles. */

          PrClose();
          HUnlock((Handle)printStuff);
          DisposHandle((Handle)printStuff);
     }
     else
          ErrorHandler();
}

short ErrorHandler()

/* Obviously a do-nothing routine for now. */
{
     ;
}

/* C routines for HyperCard callbacks */

#include <XCmdGlue.inc.c>