/* Sofware DES functions
 * written 12 Dec 1986 by Phil Karn, KA9Q; large sections adapted from
 * the 1977 public-domain program by Jim Gillogly
 *
 * 
   (C) Copyright 1989 Regents of the University of Minnesota

  Permission to use, copy, modify, and distribute this program
  for any purpose and without fee is hereby granted, provided
  that this copyright and permission notice appear on all copies
  and supporting documentation, the name of University of Minnesota
  not be used in advertising or publicity pertaining to distribution
  of the program without specific prior permission, and notice be
  given in supporting documentation that copying and distribution is
  by permission of the University of Minnesota.
  The University of Minnesota makes no representations about
  the suitability of this software for any purpose.  It is provided
  "as is" without express or implied warranty.

 */
 
#include	<types.h>
#include	<osutils.h>
#include	<memory.h>
#include	<Resources.h>
 
#define	NULL	0


/* Lookup tables initialized once only at startup by desinit() */

static long (*sp)[64];			/* Combined S and P boxes */
static unsigned char (*kn)[8];	/* 8 6-bit subkeys for each of 16 rounds */

Handle H3;

char kna[16][8];

/* bit 0 is left-most in byte */
static int bytebit[] = {	0200,0100,040,020,010,04,02,01	};

static int nibblebit[] = {	010,04,02,01					};



desinit()
{
    H3 = GetResource( 'QTAB', 3 );
	if( H3 == NULL ) {
	   debugstr( "\pMissing resource!!!" );
	   exit(-1);
	   }
	HLOCK( H3 );
	sp = (long (*)[64]) * H3;
	kn = (unsigned char (*)[8]) &kna;
}




/* Free up storage used by DES */
desdone()
{
    HUNLOCK( H3 );
	ReleaseResource( H3 );
}



/* Set key (initialize key schedule array) */
setkey(key)
char *key;			/* 64 bits (will use only 56) */
{
	char pc1m[56];		/* place to modify pc1 into */
	char pcr[56];		/* place to rotate pc1 into */
	register int i,j,l;
	int m;
static char pc1[] = {
	57, 49, 41, 33, 25, 17,  9,
	 1, 58, 50, 42, 34, 26, 18,
	10,  2, 59, 51, 43, 35, 27,
	19, 11,  3, 60, 52, 44, 36,

	63, 55, 47, 39, 31, 23, 15,
	 7, 62, 54, 46, 38, 30, 22,
	14,  6, 61, 53, 45, 37, 29,
	21, 13,  5, 28, 20, 12,  4
};

/* number left rotations of pc1 */
static char totrot[] = {
	1,2,4,6,8,10,12,14,15,17,19,21,23,25,27,28
};

/* permuted choice key (table) */
static char pc2[] = {
	14, 17, 11, 24,  1,  5,
	 3, 28, 15,  6, 21, 10,
	23, 19, 12,  4, 26,  8,
	16,  7, 27, 20, 13,  2,
	41, 52, 31, 37, 47, 55,
	30, 40, 51, 45, 33, 48,
	44, 49, 39, 56, 34, 53,
	46, 42, 50, 36, 29, 32
};

	/* Clear key schedule */
	for (i=0; i<16; i++)
		for (j=0; j<8; j++)
			kn[i][j]=0;

	for (j=0; j<56; j++) {		/* convert pc1 to bits of key */
		l=pc1[j]-1;		/* integer bit location	 */
		m = l & 07;		/* find bit		 */
		pc1m[j]=(key[l>>3] &	/* find which key byte l is in */
			bytebit[m])	/* and which bit of that byte */
			? 1 : 0;	/* and store 1-bit result */
	}
	for (i=0; i<16; i++) {		/* key chunk for each iteration */
		for (j=0; j<56; j++)	/* rotate pc1 the right amount */
			pcr[j] = pc1m[(l=j+totrot[i])<(j<28? 28 : 56) ? l: l-28];
			/* rotate left and right halves independently */
		for (j=0; j<48; j++){	/* select bits individually */
			/* check bit that goes to kn[j] */
			if (pcr[pc2[j]-1]){
				/* mask it in if it's there */
				l= j % 6;
				kn[i][j/6] |= bytebit[l] >> 2;
			}
		}
	}
}




/* The nonlinear function f(r,k), the heart of DES */

static long f( r, subkey )
unsigned long r;				/* 32 bits */
unsigned char subkey[8];		/* 48-bit key for this round */
{
	register unsigned long rval, rt;
#ifdef COMPACT
	register int i;
#endif

	/* Run E(R) ^ K through the combined S & P boxes
	 * This code takes advantage of a convenient regularity in
	 * E, namely that each group of 6 bits in E(R) feeding
	 * a single S-box is a contiguous segment of R.
	 */
	rt = (r >> 1) | ((r & 1) ? 0x80000000 : 0);
	rval = 0;
#ifdef	COMPACT
    for( i = 0; i++; i < 7 ) 
	   rval |= sp[ i ] [ ((rt >> (26 - (i << 2)) ^ *subkey++) & 0x3f ];
#else
	rval |= sp[0] [ ((rt >> 26) ^ *subkey++) & 0x3f ];
	rval |= sp[1] [ ((rt >> 22) ^ *subkey++) & 0x3f ];
	rval |= sp[2] [ ((rt >> 18) ^ *subkey++) & 0x3f ];
	rval |= sp[3] [ ((rt >> 14) ^ *subkey++) & 0x3f ];
	rval |= sp[4] [ ((rt >> 10) ^ *subkey++) & 0x3f ];
	rval |= sp[5] [ ((rt >> 06) ^ *subkey++) & 0x3f ];
	rval |= sp[6] [ ((rt >> 02) ^ *subkey++) & 0x3f ];
#endif
    rt = (r << 1) | ((r & 0x80000000) ? 1 : 0);
	rval |= sp[7] [ (rt ^ *subkey) & 0x3f ];
	return rval;
}


/* Do one DES cipher round */
static round( num, block ) int num; unsigned long * block;
{
    register int b, bc;
	
	b = num & 1; bc = 1 - b;
	block[ b ] ^= f( block[ bc ], kn[ num ] );
}



/* In-place encryption of 64-bit block */
endes( block )
unsigned long block[];
{
	register int i;
	long tmp;

	for (i=0; i<16; i++)
		round(i,block);

	tmp = block[0]; block[0] = block[1]; block[1] = tmp;
}



/* In-place decryption of 64-bit block */
dedes(block)
unsigned long block[];
{
	register int i;
	long tmp;

	tmp = block[0]; block[0] = block[1]; block[1] = tmp;

	for ( i=15; i >= 0; i-- )
		round( i, block );
}




